/*
 * tiny13_tail_light
 *  ATtiny13     .
 * 
 */
 
#define F_CPU 600000UL
#include <stdint.h>
#include <avr/io.h>
#include <avr/interrupt.h>

//  /:
#define PWM 	PB0	//       (OC0A)
#define LED 	PB3	//   -    
#define BUTTON 	PB4	//    (PCINT4)
#define BATTERY	PB2	//     (ADC1)

// :
#define SHORT_PRESS		22		//     (~75)
#define LONG_PRESS		750		//     (~3)
//  :
#define BATTERY_ALARM	216		//    
#define BATTERY_LOW		191		//   

//  :
#define CYCLE_EVEN		1		//   
#define READY_TO_SLEEP	2		//    
#define SLEEP_MODE		4		//   (power-down)

#define BUTTON_PRESSED	(!((PINB >> BUTTON) & 1))	//  

static volatile uint8_t		mode;			//  
static volatile uint8_t		state;			//  
static volatile uint8_t		pwmValue;		//     
static volatile uint16_t	buttonCounter;	//    
static volatile uint16_t	ledCounter;		//    

void power_down(void);						//    


int main(void)
{
	//   /:
//	DDRB  = 0b00011101; //
//	PORTB = 0b00100000; //
	
	// :
	DDRB  |=  (1 << PWM) | (1 << LED);			//  = 1
	PORTB &= ~((1 << PWM) | (1 << LED));		//    = 0
	// :
	DDRB  &= ~((1 << BUTTON) | (1 << BATTERY));	//  = 0
	PORTB |=   (1 << BUTTON);					//  = 1
	
	//   T0     :
	TCCR0A	= 0b00000011;	//  Fast-PWM,     
	TCCR0B	= 0b00000010;	//    CLK/8: F=600000/8/256=292,96875, T=3,4
	TCNT0	= 0;			//   
	TIMSK0	= 0b00000010;	//     
	
	//      :
	DIDR0  = 0b00101111;	//        PCINT4
	GIMSK  = 0b00100000;	//    PCINT
	PCMSK  = 0b00010000;	//       PCINT4
	ACSR   = 0b10000000;	//  
	ADCSRB = 0b00000100;	//      
	power_down();			//        
		
	sei(); //  
	
	//  
	while(1)
	{
		asm volatile ("sleep");	//         
	}
}


//    
void power_down(void)
{
	state		= SLEEP_MODE;	//    
	ledCounter	= 0;			//   
	OCR0A 		= pwmValue;		//        
	DIDR0		= 0b00101111;	//        PCINT4
	ADCSRA		= 0;			//  
	ADMUX		= 0b00100001;	//    1.1,    ,   ADC1
	PRR			= 0b00000011;	//     
	MCUCR		= 0b00110000;	//  ,  power-down
}


//   PCINT0 ( )
ISR(PCINT0_vect)
{
	if ( state == SLEEP_MODE )
	{
		//     :
		DIDR0	= 0b00100110;	//    
		PRR   	= 0;			//     
		ADMUX	= 0b01100001;	//    1.1,    ,   ADC1
		ADCSRA	= 0b10100011;	//  , -,   CLK/8 (600000/8=75)
		MCUCR	= 0b00100000;	//  ,  idle
	}
}


//      T0
ISR(TIM0_OVF_vect)
{	
	//    :
	if ( BUTTON_PRESSED ) //   
	{
		if ( buttonCounter < LONG_PRESS )
			buttonCounter++;		//  ~3.
		else if ( state < SLEEP_MODE )
		{
			TCCR0A = 0b00000011;	//  Fast-PWM,     OC0A
			PORTB &= ~(1 << LED);	//   
			state = READY_TO_SLEEP;	//      
		}
	}
	else //   
	{
		if ( state & SLEEP_MODE )
			state &= ~SLEEP_MODE;		//    
		else if ( state == READY_TO_SLEEP ) //       
		{
			power_down();				//     
			return;
		}
		else if ( buttonCounter >= SHORT_PRESS && ADCH > BATTERY_LOW ) //       ~75
		{
			//  :
			ledCounter	= 0;		//   
			PORTB &= ~(1 << LED);	//   
			if ( mode < 5 )	mode++;
			else			mode = 0;
			if		( mode == 3 ) //  4
				pwmValue = 192;		//   25%
			else if	( mode == 4 ) //  5
				pwmValue = 128;		//   50%
			else if	( mode == 5 ) //  6
				pwmValue = 64;		//   75%
			else //  1-3
				pwmValue = 0;		//   100%
			OCR0A = pwmValue;		//        
		}
		buttonCounter = 0;	//     
	}
	
	//  :
	if ( state != READY_TO_SLEEP )
	{	
		if ( ADCH <= BATTERY_LOW && ledCounter < 200 ) //   
		{
			TCCR0A	= 0b00000011;	//  Fast-PWM,     OC0A
			ledCounter = 200;
		}
		else //  
		{
			if		( ledCounter == 0 )		//  
				TCCR0A = 0b11000011;		//  Fast-PWM,     OC0A
				
			//   :
			if		( mode == 0 ) //  1
			{
				if ( ledCounter == 75 )
					TCCR0A = 0b00000011;	//  Fast-PWM,     OC0A
			}
			else if	( mode == 1 ) //  2
			{
				if ( ledCounter == 40 )
					TCCR0A = 0b11000011;	//  Fast-PWM,     OC0A
				else if ( ledCounter == 20 || ledCounter == 60 )
					TCCR0A = 0b00000011;	//  Fast-PWM,     OC0A
			}
			else if	( mode == 2 ) //  3
			{
				if ( ledCounter % 4 == 0 )	//    
				{
					if		( (state & CYCLE_EVEN) && (OCR0A < 251) )
						OCR0A += 5;				//   
					else if	( !(state & CYCLE_EVEN) && (OCR0A > 4) )
						OCR0A -= 5;				//   
					else
						state ^= CYCLE_EVEN;	//   
				}
			}
		}

		//     :
		if ( ledCounter == 200 && ADCH < BATTERY_ALARM )
			PORTB |= (1 << LED);		//   
		
		if ( ledCounter < 299 )
			ledCounter++;				//    
		else
		{
			ledCounter = 0;				//    
			PORTB &= ~(1 << LED);		//   
			
			if ( ADCH <= BATTERY_LOW )	//   
			{
				TCCR0A = 0b00000011;	//  Fast-PWM,     OC0A
				state = READY_TO_SLEEP;	//      
			}
		}
	}
}